@@ -4,12 +4,14 @@
  * \brief synchronize the tree in all distributed nodes
  */
 #include <xgboost/tree_updater.h>
-#include <vector>
-#include <string>
-#include <limits>
 
-#include "xgboost/json.h"
+#include <limits>
+#include <string>
+#include <vector>
+
+#include "../collective/communicator-inl.h"
 #include "../common/io.h"
+#include "xgboost/json.h"
 
 namespace xgboost {
 namespace tree {
@@ -20,8 +22,9 @@ DMLC_REGISTRY_FILE_TAG(updater_sync);
  * \brief syncher that synchronize the tree in all distributed nodes
  * can implement various strategies, so far it is always set to node 0's tree
  */
-class TreeSyncher: public TreeUpdater {
+class TreeSyncher : public TreeUpdater {
  public:
+  explicit TreeSyncher(GenericParameter const* tparam) : TreeUpdater(tparam) {}
   void Configure(const Args&) override {}
 
   void LoadConfig(Json const&) override {}
@@ -31,20 +34,20 @@ class TreeSyncher: public TreeUpdater {
     return "prune";
   }
 
-  void Update(HostDeviceVector<GradientPair>* ,
-              DMatrix*,
-              const std::vector<RegTree*> &trees) override {
-    if (rabit::GetWorldSize() == 1) return;
+  void Update(HostDeviceVector<GradientPair>*, DMatrix*,
+              common::Span<HostDeviceVector<bst_node_t>> /*out_position*/,
+              const std::vector<RegTree*>& trees) override {
+    if (collective::GetWorldSize() == 1) return;
     std::string s_model;
     common::MemoryBufferStream fs(&s_model);
-    int rank = rabit::GetRank();
+    int rank = collective::GetRank();
     if (rank == 0) {
       for (auto tree : trees) {
         tree->Save(&fs);
       }
     }
     fs.Seek(0);
-    rabit::Broadcast(&s_model, 0);
+    collective::Broadcast(&s_model, 0);
     for (auto tree : trees) {
       tree->Load(&fs);
     }
@@ -52,9 +55,7 @@ class TreeSyncher: public TreeUpdater {
 };
 
 XGBOOST_REGISTER_TREE_UPDATER(TreeSyncher, "sync")
-.describe("Syncher that synchronize the tree in all distributed nodes.")
-.set_body([](ObjInfo) {
-    return new TreeSyncher();
-  });
+    .describe("Syncher that synchronize the tree in all distributed nodes.")
+    .set_body([](GenericParameter const* tparam, ObjInfo) { return new TreeSyncher(tparam); });
 }  // namespace tree
 }  // namespace xgboost
