@@ -1,5 +1,5 @@
 /**
- * Copyright 2014-2022 by XGBoost Contributors
+ * Copyright 2014-2023, XGBoost Contributors
  */
 #include <dmlc/common.h>
 #include <dmlc/omp.h>
@@ -12,7 +12,6 @@
 #include <cstring>
 #include <sstream>
 #include <string>
-#include <utility>
 #include <vector>
 
 #include "../../src/c_api/c_api_error.h"
@@ -26,22 +25,24 @@
 #define R_API_BEGIN()                           \
   GetRNGstate();                                \
   try {
+
 /*!
  * \brief macro to annotate end of api
  */
-#define R_API_END()                             \
-  } catch(dmlc::Error& e) {                     \
-    PutRNGstate();                              \
-    error(e.what());                            \
-  }                                             \
+#define R_API_END()           \
+  }                           \
+  catch (dmlc::Error & e) {   \
+    PutRNGstate();            \
+    Rf_error("%s", e.what()); \
+  }                           \
   PutRNGstate();
 
 /*!
  * \brief macro to check the call.
  */
-#define CHECK_CALL(x)                           \
-  if ((x) != 0) {                               \
-    error(XGBGetLastError());                   \
+#define CHECK_CALL(x)                  \
+  if ((x) != 0) {                      \
+    Rf_error("%s", XGBGetLastError()); \
   }
 
 using dmlc::BeginPtr;
@@ -116,11 +117,25 @@ XGB_DLL SEXP XGDMatrixCreateFromMat_R(SEXP mat, SEXP missing, SEXP n_threads) {
   std::vector<float> data(nrow * ncol);
   int32_t threads = xgboost::common::OmpGetNumThreads(asInteger(n_threads));
 
-  xgboost::common::ParallelFor(nrow, threads, [&](xgboost::omp_ulong i) {
-    for (size_t j = 0; j < ncol; ++j) {
-      data[i * ncol + j] = is_int ? static_cast<float>(iin[i + nrow * j]) : din[i + nrow * j];
-    }
-  });
+  if (is_int) {
+    xgboost::common::ParallelFor(nrow, threads, [&](xgboost::omp_ulong i) {
+      for (size_t j = 0; j < ncol; ++j) {
+        auto v = iin[i + nrow * j];
+        if (v == NA_INTEGER) {
+          data[i * ncol + j] = std::numeric_limits<float>::quiet_NaN();
+        } else {
+          data[i * ncol + j] = static_cast<float>(v);
+        }
+      }
+    });
+  } else {
+    xgboost::common::ParallelFor(nrow, threads, [&](xgboost::omp_ulong i) {
+      for (size_t j = 0; j < ncol; ++j) {
+        data[i * ncol + j] = din[i + nrow * j];
+      }
+    });
+  }
+
   DMatrixHandle handle;
   CHECK_CALL(XGDMatrixCreateFromMat_omp(BeginPtr(data), nrow, ncol,
                                         asReal(missing), &handle, threads));
@@ -249,15 +264,53 @@ XGB_DLL SEXP XGDMatrixSetInfo_R(SEXP handle, SEXP field, SEXP array) {
   return R_NilValue;
 }
 
+XGB_DLL SEXP XGDMatrixSetStrFeatureInfo_R(SEXP handle, SEXP field, SEXP array) {
+  R_API_BEGIN();
+  size_t len{0};
+  if (!isNull(array)) {
+    len = length(array);
+  }
+
+  const char *name = CHAR(asChar(field));
+  std::vector<std::string> str_info;
+  for (size_t i = 0; i < len; ++i) {
+    str_info.emplace_back(CHAR(asChar(VECTOR_ELT(array, i))));
+  }
+  std::vector<char const*> vec(len);
+  std::transform(str_info.cbegin(), str_info.cend(), vec.begin(),
+                 [](std::string const &str) { return str.c_str(); });
+  CHECK_CALL(XGDMatrixSetStrFeatureInfo(R_ExternalPtrAddr(handle), name, vec.data(), len));
+  R_API_END();
+  return R_NilValue;
+}
+
+XGB_DLL SEXP XGDMatrixGetStrFeatureInfo_R(SEXP handle, SEXP field) {
+  SEXP ret;
+  R_API_BEGIN();
+  char const **out_features{nullptr};
+  bst_ulong len{0};
+  const char *name = CHAR(asChar(field));
+  XGDMatrixGetStrFeatureInfo(R_ExternalPtrAddr(handle), name, &len, &out_features);
+
+  if (len > 0) {
+    ret = PROTECT(allocVector(STRSXP, len));
+    for (size_t i = 0; i < len; ++i) {
+      SET_STRING_ELT(ret, i, mkChar(out_features[i]));
+    }
+  } else {
+    ret = PROTECT(R_NilValue);
+  }
+  R_API_END();
+  UNPROTECT(1);
+  return ret;
+}
+
 XGB_DLL SEXP XGDMatrixGetInfo_R(SEXP handle, SEXP field) {
   SEXP ret;
   R_API_BEGIN();
   bst_ulong olen;
   const float *res;
-  CHECK_CALL(XGDMatrixGetFloatInfo(R_ExternalPtrAddr(handle),
-                                   CHAR(asChar(field)),
-                                 &olen,
-                                 &res));
+  CHECK_CALL(XGDMatrixGetFloatInfo(R_ExternalPtrAddr(handle), CHAR(asChar(field)), &olen, &res));
   ret = PROTECT(allocVector(REALSXP, olen));
   for (size_t i = 0; i < olen; ++i) {
     REAL(ret)[i] = res[i];
